/*
 * Copyright (c) 2009-2017 Satlab Aps <satlab@satlab.com>
 * All rights reserved. Do not distribute without permission.
 */

#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <inttypes.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <util/crc32c.h>

uint32_t btp_crc32c_push(uint8_t *data, size_t length, uint32_t crc)
{
	return crc32c_update(crc, data, length);
}

uint32_t btp_crc32c_memory(uint8_t *data, uint32_t length)
{
	return crc32c(data, length);
}

uint32_t btp_crc32c_fd(int fd, off_t offset)
{
	int i;
	uint8_t buf[256];
	uint32_t crc, size, remain, bytes;

	size = lseek(fd, 0, SEEK_END) - offset;
	if (lseek(fd, offset, SEEK_SET) < 0) {
		printf("Failed to lseek before CRC32\r\n");
		return 0;
	}

	crc = 0;
	for (i = 0; size && i < (size + sizeof(buf) - 1) / sizeof(buf); i++) {
		remain = size - i * sizeof(buf);
		bytes = remain >= sizeof(buf) ? sizeof(buf) : remain;

		if (read(fd, buf, bytes) < bytes) {
			printf("Failed to read file for CRC32\r\n");
			return 0;
		}

		crc = btp_crc32c_push(buf, bytes, crc);
	}

	if (lseek(fd, 0, SEEK_SET) < 0) {
		printf("Failed to lseek after CRC32\r\n");
		return 0;
	}

	return crc;
}
